"Use strict";
import powerbi from "powerbi-visuals-api";
import {ISelectorManager,ISelectorManagerConstructor, IFilterManager} from "../visual/visualInterfaces";
import * as d3 from "d3";

type Selection<T extends d3.BaseType> = d3.Selection<T, any, any, any>;
/*
负责页面上选择器的展示
*/
export class SelectorManager implements ISelectorManager{
    public selectorContainer : Selection<HTMLElement>;
    private selector:ISelector;
    public filterManager:IFilterManager;
    private categories: powerbi.DataViewCategoryColumn;
    private defaultSelect: powerbi.DataViewValueColumn;
    private defaultStart: powerbi.DataViewValueColumn;
    private defaultEnd: powerbi.DataViewValueColumn;
    constructor(selectorContainer:Selection<HTMLElement>,filterManager:IFilterManager){
        console.debug("selectorManager constructor start");
        this.selectorContainer=selectorContainer;
        this.filterManager=filterManager;
    }
    public switchSelector<T extends ISelector>(classSelector:new ()=>T){
        let newSelector=new classSelector();
        this.selector?.dispose();
        this.selector=newSelector;
    }
    updateData(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn){


        if(!this.selector){
            let fieldType=categories.source.type;
            if(fieldType.text){
                this.selector=new DropDownSelector(this);
            }else if(fieldType.numeric){
                this.selector=new DropDownSelector(this);
            }else if(fieldType.dateTime){
                this.selector=new CalendarSelector(this);
            }else{
                throw "only receive text/num/datetime field";
            }
        }
        

        this.selector.update(categories, defaultSelect, defaultStart, defaultEnd);
    };
    public filterStringField(selection:string[]){

    };
}

interface ISelector{
    update(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn);
    dispose():void;
}

abstract class Selector implements ISelector{
    constructor(manager: ISelectorManager){
        console.debug("Abstract selector:","constructor start");
        this.manager=manager;
        this.createView();
    }
    protected view:Selection<HTMLElement>;
    protected abstract createView();
    public dispose(){
        this.view?.remove();
    }
    protected readonly manager:ISelectorManager;
    public abstract update(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn);
}
class DropDownSelector extends Selector{
    protected createView() {
        console.debug('dropDownViewManager','createView start');
        this.view=this.manager.selectorContainer.append("select").classed("dropDown-selector",true).classed("selector",true);
        console.debug('dropDownViewManager','createView end');
    }
    public update(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn) {
        console.debug('dropDownViewManager','update start');
        let appending=this.view.selectAll("option").data(categories.values);
        appending.enter().append("option").text(function(d){return d.toString();});
        appending.exit().remove();
        console.debug("appending:",appending);
        let filterManager:IFilterManager=this.manager.filterManager;
        (<Selection<HTMLSelectElement>>this.view).on("input change",function(){
            console.debug('DropDownSelector:',"input change");
            console.debug('this',this);
            console.debug('filterManager:',filterManager);
            let selection:string[]=[];
            for(let i=0;i<this.selectedOptions.length;i++){
                let option=this.selectedOptions.item(i);
                if(option.selected){
                    console.debug("option selected",option);
                    console.debug('option.text:',option.text);
                    selection.push(option.text);
                }
            }
            if(selection.length==0){
                
            }else{
                console.debug("selection.length",selection.length);
                filterManager.filterStringField(selection);
            }
            console.debug("this",this.selectedOptions);
        });
        console.debug('dropDownViewManager','update end');
    }
}
class ListSelector extends Selector{
    protected createView() {
        throw new Error("Method not implemented.");
    }
    public update(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn) {
        console.debug('ListSelector','updateView');
        let dropDownSelector=this.manager.selectorContainer.append("select").classed("listSelector");
    }
}
class CalendarSelector extends Selector{
    protected createView() {
        throw new Error("Method not implemented.");
    }
    public update(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn) {
        console.debug('calendarViewManager','updateView');
        let dropDownSelector=this.manager.selectorContainer.append("select").classed("listSelector");
    }
}